/* Copyright (c) 2003 Andrew Stikheev. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * $Id: viewstat.c,v 1.10 2003/06/01 18:34:03 sand Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <fcntl.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

#include <err.h>
#include <errno.h>
#include <string.h>
#include "statistics.h"

static int verbose = 0;

static void PutOutStatistics	( int fd );
static void RemoveStatistics	( int fd, char * );
static void ParseArgs		( int argc, char *argv[]  );
static void Usage		( );
static void Quit		( const char * );
////////////////////////////////////////////////////////////////////////////////
int main( int  argc, char *argv[] )
{
	ParseArgs( argc , argv );
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
void Quit (const char* msg)             
{
        warn (msg);
        exit (1);
}
////////////////////////////////////////////////////////////////////////////////
static void RemoveStatistics(int fd,  char *ipAddr )
{
	clusterItem		temp;
	signed char		i,resetAll;
	struct in_addr		ip_src;
	long			shift;
	in_addr_t		nullIP = 0;
	
	resetAll = !strcmp ( ipAddr,"all" );
		
	while ( sizeof (clusterItem) == read(fd,&temp,sizeof(clusterItem)) ) {

        	for( i = 0; i < temp.totalItems; i++ ) {

			if ( !temp.item[i].srcIP ) continue;

                	ip_src.s_addr = temp.item[i].srcIP;
				
			if ( resetAll || 
			     !strcmp( ipAddr,(char *)inet_ntoa(ip_src)) ) {

				shift = (long) &(temp.item[i].srcIP) - 
					(long) &(temp) - sizeof( clusterItem ); 
				if ( lseek (fd,shift,SEEK_CUR ) == -1 )
					Quit("lseek");

				if ( write (fd, &nullIP, sizeof(in_addr_t))
					!=  sizeof ( in_addr_t )  )
					 Quit("write");

				shift = -shift - sizeof( in_addr_t );

				if ( lseek(fd, shift,SEEK_CUR) == -1 )
					Quit("lseek");

				if ( verbose )
				printf("%s removed\n",inet_ntoa(ip_src));
			} 

		} //for 
	} //while
}
////////////////////////////////////////////////////////////////////////////////
static void PutOutStatistics( int fd )
{
	clusterItem             temp;
	struct in_addr          ip_src,ip_dst;
	long                    j=0;
	signed char             i;
 	struct stat 		filestat;
	
	if ( verbose ) {

 		if (fstat(fd,&filestat)) Quit("fstat"); 

		printf("Number of clusters = %d\n",
		(int)	filestat.st_size / sizeof (clusterItem));
	}

	while ( sizeof(clusterItem) == read( fd, &temp, sizeof(clusterItem)) ) {

              	for( i = 0; i < temp.totalItems; i++ ) {

			if ( !verbose && !temp.item[i].srcIP ) continue;

                       	ip_src.s_addr = temp.item[i].srcIP;
                       	ip_dst.s_addr = temp.item[i].lastDstIP;

                       	printf("%ld[%d]   %s:%d > ",
				j, i, 
				inet_ntoa(ip_src), 
				ntohs(temp.item[i].lastSrcPort)	);

                       	printf("%s:%d   %ld   %.2f\n",
				inet_ntoa(ip_dst),
                                ntohs(temp.item[i].lastDstPort),
                                temp.item[i].lastTime,
				temp.item[i].weight 		);
               	}
		j++;
//		printf("%d:%d\n",j,i);
  	}
//	printf("ok\n");
}
////////////////////////////////////////////////////////////////////////////////
static void ParseArgs(int argc, char *argv[])
{

	int ch , flag = 0, fd = -1;
	char buf[16];

	while( (ch=getopt(argc,argv,"vf:d:")) != -1)
		switch (ch) {

		case 'f':
			
 				fd = open ( optarg, O_RDWR );
        			if (fd < 0) Quit("open"); 
				break;
		case 'd':
				if (strlen(optarg) > strlen("255.255.255.255")) 
					Usage();

				strcpy(buf,optarg);				
				flag = 1;
		 		break;	 	
		case 'v': 
				verbose = 1;
				break;
		default :
				Usage();
		}

	if ( fd < 0 ) Usage(); 

	if ( flag  ) {
		
		RemoveStatistics ( fd , buf );
	} else  PutOutStatistics ( fd );

	close( fd );
}
////////////////////////////////////////////////////////////////////////////////
static void Usage()
{
	fprintf(stderr,"Usage : ./scandctl [ -d ip-address, all ] -f mfile\n");
	exit(1);
}
////////////////////////////////////////////////////////////////////////////////
